#include <iostream>
#include <vector>
#include <algorithm>
#include <cassert>

#if !defined LOCAL && !defined DEBUG
#define USE_FAST_IO
#endif

#ifdef USE_FAST_IO
#define endl '\n'
#endif

using namespace std;

struct Edge {
    int to, weight;
    explicit Edge() = default;
    Edge(int u, int w) : to(u), weight(w) {

    }
};

inline int ith_bit(int n, int bit) {
    return (n >> bit) & 1;
}

using Graph = vector<vector<Edge>>;

void fill_reachable(const Graph &graph, int v, vector<int> &used) {
    used[v] = 1;
    for (auto [u, w] : graph[v]) {
        if (!used[u]) {
            fill_reachable(graph, u, used);
        }
    }
}

void fill_topsort(const Graph &graph, int v, vector<int> &used, vector<int> &order) {
    used[v] = 1;
    for (auto [u, w] : graph[v]) {
        if (!used[u]) {
            fill_topsort(graph, u, used, order);
        }
    }
    order.push_back(v);
}

void fill_comps(const Graph &graph, int v, vector<int> &comps, int timer) {
    comps[v] = timer;
    for (auto [u, w] : graph[v]) {
        if (comps[u] == -1) {
            fill_comps(graph, u, comps, timer);
        }
    }
}

void fill_parts(int bit, const Graph &graph, int v, vector<int> &comps, vector<int> &parts) {
    parts[v] = 1;
    for (auto [u, w] : graph[v]) {
        if (comps[v] == comps[u] && ith_bit(w, bit) == 0 && parts[u] != 1) {
            fill_parts(bit, graph, u, comps, parts);
        }
    }
}


void solve() {
    int n, m; cin >> n >> m;
    Graph graph(n), rev_graph(n);
    bool topsorted = true;
    bool is_cycle = n == m;
    int max_bit = 0;
    int sum_weights = 0;
    vector<int> loops(n, -1);
    for (int i = 0; i < m; i++) {
        int v, u, w; cin >> v >> u >> w;
        v--; u--;
        is_cycle = is_cycle && (v == i) && (u == ((i + 1) % n));
        if (v == u) {
            loops[v] = max(loops[v], w);
        }
        if (w > 0) {
            max_bit = max(max_bit, 32 - __builtin_clz(w));
        }
        if (v >= u) {
            topsorted = false;
        }
        sum_weights ^= w;
        graph[v].emplace_back(u, w);
        rev_graph[u].emplace_back(v, w);
    }
    int a, b; cin >> a >> b; a--; b--;
#ifndef LOCAL
    if (topsorted) {
        vector<int> sums(n), parity(n);
        vector<bool> rb(n);
        for (int v = n - 1; v >= 0; v--) {
            if (v == b) {
                rb[v] = true;
                parity[v] = 1;
            }
            for (auto [u, w] : graph[v]) {
                if (!rb[u]) continue;
                sums[v] ^= sums[u] ^ (w * parity[u]);
                parity[v] ^= parity[u];
                rb[v] = true;
            }
        }
        cout << sums[a] << endl;
        return;
    }
#endif
    if (is_cycle) {
        int w_ab = 0;
        for (int i = a; i != b; i = ((i + 1) % n)) {
            w_ab ^= graph[i][0].weight;
        }
        if (sum_weights != 0 || w_ab != 0) {
            cout << -1 << endl;
        } else {
            cout << 0 << endl;
        }
        return;
    }
    vector<int> reachable_a(n, 0), reachable_b(n, 0);
    fill_reachable(graph, a, reachable_a);
    fill_reachable(rev_graph, b, reachable_b);
    vector<bool> deleted(n);
    for (int i = 0; i < n; i++) {
        if (!reachable_a[i] || !reachable_b[i]) {
            deleted[i] = true;
            graph[i].clear();
            rev_graph[i].clear();
        }
    }
    for (vector<Edge> &z : graph) {
        vector<Edge> new_z;
        for (auto [v, w] : z) {
            if (!deleted[v]) {
                new_z.push_back({v, w});
            }
        }
        z = new_z;
    }
    for (vector<Edge> &z : rev_graph) {
        vector<Edge> new_z;
        for (auto [v, w] : z) {
            if (!deleted[v]) {
                new_z.push_back({v, w});
            }
        }
        z = new_z;
    }

    vector<int> used(n, 0);
    used.assign(n, 0);
    vector<int> order;
    for (int i = 0; i < n; i++) {
        if (!used[i]) {
            fill_topsort(graph, i, used, order);
        }
    }
    reverse(order.begin(), order.end());
    vector<int> cmps(n, -1);
    vector<int> starts;
    int timer = 0;
    for (int i : order) {
        if (cmps[i] == -1) {
            starts.push_back(i);
            fill_comps(rev_graph, i, cmps, timer);
            timer++;
        }
    }
    vector<int> cmps_sizes(timer, 0);
    vector<int> cmps_preds(timer);

    for (int v = 0; v < n; v++) {
        cmps_sizes[cmps[v]]++;
        cmps_preds[cmps[v]] = v;
    }
    int ans = 0;
    for (int bit = 0; bit < max_bit; bit++) {
        vector<int> parts(n);
        for (int start : starts) {
            fill_parts(bit, graph, start, cmps, parts);
        }
        for (int v = 0; v < n; v++) {
            for (auto [u, w] : graph[v]) {
                if (cmps[v] == cmps[u] && parts[v] == parts[u] && ith_bit(w, bit) != 0) {
                    cout << -1 << endl;
                    return;
                }
                if (cmps[v] == cmps[u] && parts[v] != parts[u]) {
                    assert(ith_bit(w, bit) == 1);
                }
            }
        }
        Graph cond_graph(timer);
        for (int v = 0; v < n; v++) {
            for (auto [u, w] : graph[v]) {
                int a = cmps[v], b = cmps[u];
                if (a != b) {
                    int new_w = parts[u] ^ parts[v] ^ ith_bit(w, bit);
                    cond_graph[a].emplace_back(b, new_w);
                }
            }
        }
        vector<int> only_way_b(timer, -1);
        vector<bool> rbcmp(timer);
        for (int i = timer - 1; i >= 0; i--) {
            if (i == cmps[b]) {
                rbcmp[i] = true;
                only_way_b[i] = 0;
            }
            for (auto [j, w] : cond_graph[i]) {
                if (!rbcmp[j]) continue;

                rbcmp[i] = true;
                if (only_way_b[i] != (w ^ only_way_b[j]) && only_way_b[i] != -1) {
                    only_way_b[i] = -2;
                    break;
                }
                only_way_b[i] = w ^ only_way_b[j];
            }
        }
        vector<int> only_way_a(timer, -1);
        vector<bool> racmp(timer);
        for (int i = 0; i < timer; i++) {
            if (i == cmps[a]) {
                racmp[i] = true;
                only_way_a[i] = 0;
            }
            for (auto [j, w] : cond_graph[i]) {
                if (!racmp[j]) continue;
                racmp[i] = true;
                if (only_way_a[i] != (w ^ only_way_a[j]) && only_way_a[i] != -1) {
                    only_way_a[i] = -2;
                    break;
                }
                only_way_a[i] = w ^ only_way_a[j];
            }
        }
        for (int i = 0; i < timer; i++) {
            if (racmp[i] && rbcmp[i] && (cmps_sizes[i] > 1 || loops[cmps_preds[i]] != -1)) {
                if (only_way_a[i] == -2 || only_way_b[i] == -2) {
                    cout << -1 << endl;
                    return;
                }
                if ((only_way_a[i] ^ only_way_b[i] ^ parts[a] ^ parts[b]) != 0) {
                    cout << -1 << endl;
                    return;
                }
            }
        }
        if (cmps_sizes[cmps[a]] > 1 || cmps_sizes[cmps[b]] > 1 || loops[a] != -1 || loops[b] != -1) {
            continue;
        }

        vector<bool> new_deleted(n);
        for (int i = 0; i < n; i++) {
            new_deleted[i] = deleted[i] || (cmps_sizes[cmps[i]] > 1 && loops[i] == -1);
        }
        
        Graph new_graph(n);
        for (int v = 0; v < n; v++) {
            if (new_deleted[v]) continue;
            const vector<Edge> &z = graph[v];
            vector<Edge> new_z;
            for (auto [v, w] : z) {
                if (!new_deleted[v]) {
                    new_z.push_back({v, w});
                }
            }
            new_graph[v] = new_z;
        }
        vector<int> new_used(n);
        vector<int> new_order;
        fill_topsort(new_graph, a, new_used, new_order);
        // reverse(new_order.begin(), new_order.end());
        vector<int> sums(n), parity(n);
        vector<bool> rb(n);
        for (int v : new_order) {
            if (v == b) {
                rb[v] = true;
                parity[v] = 1;
            }
            for (auto [u, w] : new_graph[v]) {
                if (!rb[u]) continue;
                rb[v] = true;
                sums[v] ^= sums[u] ^ (parity[u] * ith_bit(w, bit));
                parity[v] ^= parity[u];
            }
        }
        ans ^= sums[a] << bit;
    }
    cout << ans << endl;
}

int main() {
#ifdef USE_FAST_IO
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int t; cin >> t;
    while (t--) {
        solve();
    }
}
