#include <iostream>
#include <vector>
#include <algorithm>

#if !defined LOCAL && !defined DEBUG
#define USE_FAST_IO
#endif

#ifdef USE_FAST_IO
#define endl '\n'
#endif

using namespace std;
using Graph = vector<vector<int>>;

struct Segment {
    int l, r;
    explicit Segment() = default;
    Segment(int l_, int r_) : l(l_), r(r_) {

    }
    int ways() const {
        return r - l + 1;
    }
};

void intersect(Segment &a, const Segment &b) {
    a.l = max(a.l, b.l);
    a.r = min(a.r, b.r);
}

bool broken(const Segment &s) {
    return s.l > s.r;
}

const int MOD = 1'000'000'007;

int modmul(int a, int b) {
    return (a * 1ll * b) % MOD;
}

int binmodpow(int a, int n) {
    if (n == 0) {
        return 1;
    }
    if (n % 2 == 1) {
        return modmul(a, binmodpow(a, n - 1));
    }
    return binmodpow(modmul(a, a), n / 2);
}

int modrev(int a) {
    return binmodpow(a, MOD - 2);
}

struct DSU {
    vector<int> parents;
    vector<int> sizes;
    vector<Segment> segs;
    bool is_broken;
    int product = 1;

    DSU(const vector<Segment> &segs) {
        int n = segs.size();
        this->segs = segs;
        sizes.assign(n, 1);
        parents.assign(n, -1);
        is_broken = false;
        for (const Segment &s : segs) {
            product = modmul(product, s.ways());
        }
    }

    int find(int v) {
        if (parents[v] == -1) {
            return v;
        }
        return parents[v] = find(parents[v]);
    }

    void merge(int v, int u) {
        v = find(v);
        u = find(u);
        if (v == u) {
            return;
        }
        if (sizes[v] > sizes[u]) {
            swap(v, u);
        }
        product = modmul(product, modrev(segs[u].ways()));
        product = modmul(product, modrev(segs[v].ways()));
        intersect(segs[u], segs[v]);
        if (broken(segs[u])) {
            is_broken = true;
            return;
        }
        sizes[u] += sizes[v];
        parents[v] = u;
        product = modmul(product, segs[u].ways());
    }
};

void fill_depths(const Graph &tree, int v, int p, vector<int> &depths) {
    if (p == -1 || p == v) {
        depths[v] = 0;
    } else {
        depths[v] = 1 + depths[p];
    }
    for (int u : tree[v]) {
        if (u != p) {
            fill_depths(tree, u, v, depths);
        }
    }
}

vector<int> get_path(int v, int u, int dv, int du, const vector<int> &parents) {
    vector<int> left_half, right_half;
    while (dv < du) {
        right_half.push_back(u);
        u = parents[u];
        du--;
    }
    if (v == u) {
        right_half.push_back(u);
        reverse(right_half.begin(), right_half.end());
        return right_half;
    }
    while (dv > du) {
        left_half.push_back(v);
        v = parents[v];
        dv--;
    }
    if (v == u) {
        left_half.push_back(v);
        return left_half;
    }
    while (v != u) {
        left_half.push_back(v);
        right_half.push_back(u);
        v = parents[v]; u = parents[u];
    }
    left_half.push_back(v);
    reverse(right_half.begin(), right_half.end());
    for (int i : right_half) {
        left_half.push_back(i);
    }
    return left_half;
}

int main() {
#ifdef USE_FAST_IO
    ios_base::sync_with_stdio(false);
    cin.tie(nullptr);
#endif
    int n; cin >> n;
    vector<int> parents(n, -1);
    Graph tree(n);
    for (int i = 1; i < n; i++) {
        cin >> parents[i];
        parents[i]--;
        tree[i].push_back(parents[i]);
        tree[parents[i]].push_back(i);
    }
    vector<int> depths(n);
    fill_depths(tree, 0, -1, depths);
    vector<Segment> segs;
    for (int i = 0; i < n; i++) {
        int l, r; cin >> l >> r;
        segs.emplace_back(l, r);
    }
    DSU dsu(segs);
    int m; cin >> m;
    vector<int> answers(m);
    for (int i = 0; i < m && !dsu.is_broken; i++) {
        int a, b, c, d; cin >> a >> b >> c >> d; a--; b--; c--; d--;
        vector<int> p1 = get_path(a, b, depths[a], depths[b], parents);
        vector<int> p2 = get_path(c, d, depths[c], depths[d], parents);

        int k = p1.size();
        for (int i = 0; i < k && !dsu.is_broken; i++) {
            dsu.merge(p1[i], p2[i]);
        }
        if (!dsu.is_broken) {
            answers[i] = dsu.product;
        }
    }
    for (int i : answers) {
        cout << i << endl;
    }
}
